# ------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ------------------------------------------------------------------------------
from mssqlctl.cli.core.commands import CliCommandGroup
from mssqlctl.cli.commands.cluster.client import (
    beget,
    beget_no_check_auth,
    beget_no_check_auth_or_eula
)


def load_command_table(self, _):

    def hierarchical_output(command_result):
        """
        Callback for formatting complex custom-output.
        :parm_am command_result: The command's high-level result object.
        :return: Complex BoxLayout otherwise flat json.
        """
        from mssqlctl.cli.core.layout import BoxLayout
        from mssqlctl.cli.commands.cluster.util import order_status

        try:
            raw_result = command_result.result
            result = order_status('bdc')(raw_result)
            result = result if result else order_status('services')(raw_result)
            result = result if result else order_status('resources')(raw_result)
            return BoxLayout(result, identifiers=['healthStatus', 'state'])
        except Exception as e:  # -- fallback --
            from knack.output import format_json
            return format_json(command_result)

    with CliCommandGroup(self, 'bdc',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget_no_check_auth_or_eula) as g:
        g.command('create', 'cluster_create')

    with CliCommandGroup(self, 'bdc',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget_no_check_auth) as g:
        g.command('delete', 'cluster_delete')

    with CliCommandGroup(self, 'bdc',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('upgrade', 'cluster_upgrade')

    with CliCommandGroup(self, 'bdc config',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget_no_check_auth_or_eula) as g:
        g.command('list', 'cluster_config_list')
        g.command('init', 'cluster_config_init')

    with CliCommandGroup(self, 'bdc config',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget_no_check_auth) as g:
        g.command('show', 'cluster_config_show')
        g.command('patch', 'cluster_config_patch')
        g.command('add', 'cluster_config_add')
        g.command('replace', 'cluster_config_replace')
        g.command('remove', 'cluster_config_remove')

    with CliCommandGroup(self, 'bdc debug',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget_no_check_auth) as g:
        g.command('copy-logs', 'cluster_debug_copy_logs')
        g.command('dump', 'cluster_debug_dump')

    with CliCommandGroup(self, 'bdc endpoint',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('list', 'cluster_endpoint')

    with CliCommandGroup(self, 'bdc status',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('show', 'bdc_status_show', output=hierarchical_output)

    with CliCommandGroup(self, 'bdc control status',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('show', 'bdc_control_status_show', output=hierarchical_output)

    with CliCommandGroup(self, 'bdc sql status',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('show', 'bdc_sql_status_show', output=hierarchical_output)

    with CliCommandGroup(self, 'bdc hdfs status',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('show', 'bdc_hdfs_status_show', output=hierarchical_output)

    with CliCommandGroup(self, 'bdc spark status',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('show', 'bdc_spark_status_show', output=hierarchical_output)

    with CliCommandGroup(self, 'bdc gateway status',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('show', 'bdc_gateway_status_show', output=hierarchical_output)

    with CliCommandGroup(self, 'bdc app status',
                         'mssqlctl.cli.commands.cluster.custom#{}',
                         client_factory=beget) as g:
        g.command('show', 'bdc_app_status_show', output=hierarchical_output)
